// =====================================================================
// kstream.h
// =====================================================================

// Author:         (c) 2013 Daniele Cono D'Elia, Camil Demetrescu
// License:        See the end of this file for license information
// Created:        June 21, 2013

// Last changed:   $Date: 2013/06/20 13:01:07 $
// Changed by:     $Author: demetres $
// Revision:       $Revision: 1.00 $


#ifndef __KSTREAM__
#define __KSTREAM__

#include <stdint.h>

// kstream object type
typedef struct kstream kstream;
typedef struct kf_node kf_node;

// kf node
struct kf_node {
    uint32_t id;
    uint32_t count; 
    kf_node* first_child;
    kf_node* next_sibling;
};

// kf statistics
typedef struct {
    uint32_t num_trees;      // number of tres in the forest
    uint32_t num_nodes;      // number of nodes in the forest
    uint32_t num_int_nodes;  // number of internal nodes in the forest
    uint32_t num_levels;     // number of levels of the forest
    double counter_skewness; // counter skewness, defined as maximum ratio 
                             // of a child's counter w.r.t. its parent's 
                             // counter, averaged over all internal nodes
} kf_stat;


kstream*  kstream_new(char* name, uint32_t num_ids, uint32_t k);
void      kstream_start(kstream* f);
void      kstream_emit(kstream* f, uint32_t id);
void      kstream_dump(kstream* f, char* log_file);
void      kstream_delete(kstream* f);

kf_node** kf_new(kstream* f);
void      kf_delete(kf_node** kf);
void      kf_get_stat(kf_node** kf, kf_stat* stat);
void      kf_print_stat(kf_stat* stat);

#endif


// Copyright (C) 2013 Danile Cono D'Elia, Camil Demetrescu

// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.

// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
// USA

